/*
Code stolen from http://rusanu.com/2007/04/25/reusing-conversations/
*/
-- This table associates the current connection (@@SPID)
-- with a conversation. The association key also
-- contains the conversation parameteres:
-- from service, to service, contract used
CREATE TABLE [SessionConversations] (
SPID INT NOT NULL,
FromService SYSNAME NOT NULL,
ToService SYSNAME NOT NULL,
OnContract SYSNAME NOT NULL,
Handle UNIQUEIDENTIFIER NOT NULL,
PRIMARY KEY (SPID, FromService, ToService, OnContract),
UNIQUE (Handle));
GO


-- SEND procedure. Will lookup to reuse an existing conversation
-- or start a new in case no conversation exists or the conversation
-- cannot be used
CREATE PROCEDURE [usp_Send] (
@fromService SYSNAME,
@toService SYSNAME,
@onContract SYSNAME,
@messageType SYSNAME,
@messageBody VARBINARY(MAX))
AS
BEGIN
	SET NOCOUNT ON;
	DECLARE @handle UNIQUEIDENTIFIER;
	DECLARE @counter INT;
	DECLARE @error INT;
	
	SELECT @counter = 1;
	BEGIN TRANSACTION;
	-- Will need a loop to retry in case the conversation is
	-- in a state that does not allow transmission
	WHILE (1=1)
	BEGIN
		-- Seek an eligible conversation in [SessionConversations]
		SELECT @handle = Handle
		FROM [SessionConversations]
		WHERE SPID = @@SPID
			AND FromService = @fromService
			AND ToService = @toService
			AND OnContract = @OnContract;

		IF @handle IS NULL
		BEGIN
			-- Need to start a new conversation for the current @@spid
			BEGIN DIALOG CONVERSATION @handle
			FROM SERVICE @fromService
			TO SERVICE @toService
			ON CONTRACT @onContract
			WITH ENCRYPTION = OFF;

			INSERT INTO [SessionConversations]
			(SPID, FromService, ToService, OnContract, Handle)
			VALUES
			(@@SPID, @fromService, @toService, @onContract, @handle);
		END;
		-- Attempt to SEND on the associated conversation

		SEND ON CONVERSATION @handle
		MESSAGE TYPE @messageType
		(@messageBody);

		SELECT @error = @@ERROR;
		IF @error = 0
		BEGIN
		-- Successful send, just exit the loop
			BREAK;
		END

		SELECT @counter = @counter+1;
		IF @counter > 10
		BEGIN
			 We failed 10 times in a row, something must be broken
			RAISERROR (NFailed to SEND on a conversation for more than 10 times. Error %i., 16, 1, @error) WITH LOG;
			BREAK;
		END

		-- Delete the associated conversation from the table and try again
		DELETE FROM [SessionConversations]
		WHERE Handle = @handle;

		SELECT @handle = NULL;
	END
	COMMIT;
END
GO
-- Test the procedure with some dummy paramaters. The FromService ST
-- must already exist in the database
--
exec usp_Send ST, Foobar, NDEFAULT, NDEFAULT, 001020304